@@ -19,19 +19,18 @@
 
 from projects.dataset_condensation.utils import freeze_bn
 import trojanvision
-from trojanvision.utils import summary
 import argparse
+import os
 
-from trojanvision.utils.model import weight_init
-from trojanzoo.utils.tensor import save_tensor_as_img
+from trojanzoo.utils.model import init_weights
 from trojanzoo.utils.logger import SmoothedValue
 from trojanzoo.utils.output import prints, ansi
-from trojanzoo.utils.fim import fim, fim_diag, KFAC
+from trojanzoo.utils.fim import fim, fim_diag, KFAC, EKFAC
 
 import torch
 from torch.utils.data import TensorDataset
 from torchvision import transforms
-import functools
+import torchvision.transforms.functional as F
 
 from utils import match_loss, augment, get_daparam, get_loops
 from model import ConvNet
@@ -54,7 +53,7 @@ default_args = {
 eval_args = {
     'momentum': 0.9,
     'weight_decay': 5e-4,
-    'adv_train': True,
+    'adv_train': 'pgd',
 }
 
 fgsm_args = {
@@ -101,53 +100,56 @@ if __name__ == '__main__':
     parser.add_argument('--eval_interval', type=int, default=10)
     parser.add_argument('--file_path')
     parser.add_argument('--init', default='noise', choices=['noise', 'real'],
-                        help='noise/real: initialize synthetic images from random noise or randomly sampled real images.')
+                        help='noise/real: initialize synthetic images '
+                        'from random noise or randomly sampled real images.')
     parser.add_argument('--first_term', choices=['fgsm', 'pgd'])
     parser.add_argument('--eval_adv_train', action='store_true')
-    args = parser.parse_args()
+    kwargs = parser.parse_args().__dict__
 
-    dis_metric: str = args.dis_metric
-    image_per_class: int = args.image_per_class
-    num_eval: int = args.num_eval
-    epoch_eval_train: int = args.epoch_eval_train
-    Iteration: int = args.Iteration
-    lr_img: float = args.lr_img
+    dis_metric: str = kwargs['dis_metric']
+    image_per_class: int = kwargs['image_per_class']
+    num_eval: int = kwargs['num_eval']
+    epoch_eval_train: int = kwargs['epoch_eval_train']
+    Iteration: int = kwargs['Iteration']
+    lr_img: float = kwargs['lr_img']
     outer_loop, inner_loop = get_loops(image_per_class)
+    first_term: str = kwargs['first_term']
 
-    args_dict = args.__dict__
-    args_dict.update(default_args)
-    env = trojanvision.environ.create(**args_dict)
-    dataset = trojanvision.datasets.create(**args_dict)
+    kwargs.update(default_args)
+    env = trojanvision.environ.create(**kwargs)
+    dataset = trojanvision.datasets.create(**kwargs)
 
     fgsm_dict = fgsm_args[dataset.name]
     pgd_dict = pgd_args[dataset.name]
-    args_dict['adv_train_eval_iter'] = pgd_dict['iteration']
-    args_dict['adv_train_eval_alpha'] = pgd_dict['pgd_alpha']
-    args_dict['adv_train_eval_eps'] = pgd_dict['pgd_eps']
-    model = trojanvision.models.create(dataset=dataset, **args_dict)
-    trainer = trojanvision.trainer.create(dataset=dataset, model=model, **args_dict)
+    kwargs['adv_train_eval_iter'] = pgd_dict['iteration']
+    kwargs['adv_train_eval_alpha'] = pgd_dict['pgd_alpha']
+    kwargs['adv_train_eval_eps'] = pgd_dict['pgd_eps']
+    model = trojanvision.models.create(dataset=dataset, **kwargs)
+    trainer = trojanvision.trainer.create(dataset=dataset, model=model, **kwargs)
 
     if env['verbose']:
-        summary(env=env, dataset=dataset, model=model, trainer=trainer)
+        trojanvision.summary(env=env, dataset=dataset, model=model, trainer=trainer)
 
-    if args_dict['eval_model'] is not None:
-        args_dict['model_name'] = args_dict['eval_model']
-    if args_dict['eval_norm_layer'] is not None:
-        args_dict['norm_layer'] = args_dict['eval_norm_layer']
-    args_dict.update(eval_args)
-    if args.first_term == 'fgsm':
-        args_dict['adv_train_iter'] = fgsm_dict['iteration']
-        args_dict['adv_train_alpha'] = fgsm_dict['pgd_alpha']
-        args_dict['adv_train_eps'] = fgsm_dict['pgd_eps']
-    if args.first_term == 'pgd':
-        args_dict['adv_train_iter'] = pgd_dict['iteration']
-        args_dict['adv_train_alpha'] = pgd_dict['pgd_alpha']
-        args_dict['adv_train_eps'] = pgd_dict['pgd_eps']
-    eval_model = trojanvision.models.create(dataset=dataset, **args_dict)
-    eval_trainer = trojanvision.trainer.create(dataset=dataset, model=eval_model, **args_dict)
+    if kwargs['eval_model'] is not None:
+        kwargs['model_name'] = kwargs['eval_model']
+    if kwargs['eval_norm_layer'] is not None:
+        kwargs['norm_layer'] = kwargs['eval_norm_layer']
+    if kwargs['adv_train'] == 'trades':
+        eval_args.update(adv_train='trades')
+    kwargs.update(eval_args)
+    if first_term == 'fgsm':
+        kwargs['adv_train_iter'] = fgsm_dict['iteration']
+        kwargs['adv_train_alpha'] = fgsm_dict['pgd_alpha']
+        kwargs['adv_train_eps'] = fgsm_dict['pgd_eps']
+    if first_term == 'pgd':
+        kwargs['adv_train_iter'] = pgd_dict['iteration']
+        kwargs['adv_train_alpha'] = pgd_dict['pgd_alpha']
+        kwargs['adv_train_eps'] = pgd_dict['pgd_eps']
+    eval_model = trojanvision.models.create(dataset=dataset, **kwargs)
+    eval_trainer = trojanvision.trainer.create(dataset=dataset, model=eval_model, **kwargs)
 
     if env['verbose']:
-        summary(eval_model=eval_model, eval_trainer=eval_trainer)
+        trojanvision.summary(eval_model=eval_model, eval_trainer=eval_trainer)
 
     init_fn = torch.randn if dataset.normalize else torch.rand
     image_syn = init_fn(size=(model.num_classes, image_per_class, *dataset.data_shape),
@@ -159,25 +161,25 @@ if __name__ == '__main__':
     optimizer_img.zero_grad()
     # lr_scheduler_img = torch.optim.lr_scheduler.CosineAnnealingLR(optimizer_img, T_max=Iteration * outer_loop)
 
-    transform = [transforms.ToTensor()]
+    transform = [transforms.PILToTensor(), transforms.ConvertImageDtype(torch.float)]
     if dataset.normalize and dataset.norm_par is not None:
         transform.append(transforms.Normalize(mean=dataset.norm_par['mean'], std=dataset.norm_par['std']))
-    train_set = dataset.get_full_dataset(mode='train', transform=transforms.Compose(transform))
-    # class_list = [torch.stack(dataset_to_list(dataset.get_dataset(dataset=train_set, class_list=[c]))[0])
+    train_set = dataset.get_dataset(mode='train', transform=transforms.Compose(transform))
+    # class_list = [dataset_to_tensor(dataset.get_class_subset(dataset=train_set, class_list=[c]))[0]
     #               for c in range(model.num_classes)]
     class_loader_list = [dataset.get_dataloader(mode='train', drop_last=True, num_workers=0, pin_memory=False,
-                                                dataset=dataset.get_dataset(dataset=train_set, class_list=[c]))
+                                                dataset=dataset.get_class_subset(dataset=train_set, class_list=[c]))
                          for c in range(model.num_classes)]
     iter_list = [iter(loader) for loader in class_loader_list]
 
     train_args = dict(**trainer)
-    # train_args['epoch'] = 1
-    train_args['epoch'] = inner_loop
+    # train_args['epochs'] = 1
+    train_args['epochs'] = inner_loop
     train_args['lr_scheduler'] = None
-    train_args['adv_train'] = False if args.first_term is None else True
+    train_args['adv_train'] = first_term is not None
     eval_train_args = dict(**eval_trainer)
-    eval_train_args['epoch'] = epoch_eval_train
-    eval_train_args['adv_train'] = args.eval_adv_train
+    eval_train_args['epochs'] = epoch_eval_train
+    eval_train_args['adv_train'] = kwargs['eval_adv_train']
     mean_value = [0.0]
     std_value = [1.0]
     if dataset.norm_par is not None:
@@ -192,6 +194,8 @@ if __name__ == '__main__':
     kfac = None
     if dis_metric == 'kfac':
         kfac = KFAC(model)
+    elif dis_metric == 'ekfac':
+        kfac = EKFAC(model)
 
     def get_data_fn(data, **kwargs):
         _input, _label = eval_model.get_data(data, **kwargs)
@@ -200,38 +204,34 @@ if __name__ == '__main__':
 
     def get_real_grad(img_real: torch.Tensor, lab_real: torch.Tensor,
                       adv_train: bool = False) -> list[torch.Tensor]:
-        if adv_train:
-            img_real, _ = model.pgd.optimize(_input=img_real, target=lab_real,
-                                             iteration=model.adv_train_iter,
-                                             pgd_alpha=model.adv_train_alpha,
-                                             pgd_eps=model.adv_train_eps)
-
-        if dis_metric == 'kfac':
+        if dis_metric in ['kfac', 'ekfac']:
             kfac.track.enable()
-        loss_real = model.loss(img_real, lab_real)
+        loss_fn = model.adv_loss if adv_train else model.loss
+        loss_real = loss_fn(_input=img_real, _label=lab_real)
         gw_real = list((grad.detach().clone() for grad in torch.autograd.grad(loss_real, net_parameters)))
-        if dis_metric == 'kfac':
+        if dis_metric in ['kfac', 'ekfac']:
             kfac.track.disable()
-            kfac.update_covs()
-            kfac.update_inv_covs()
+            kfac.update_stats()
         return gw_real
 
-    def get_syn_grad(img_syn: torch.Tensor, lab_syn: torch.Tensor) -> list[torch.Tensor]:
-        loss_syn = model.loss(img_syn, lab_syn)
-        gw_syn = torch.autograd.grad(loss_syn, net_parameters, create_graph=True)
+    def get_syn_grad(img_syn: torch.Tensor, lab_syn: torch.Tensor,
+                     adv_train: bool = False) -> list[torch.Tensor]:
+        loss_fn = model.adv_loss if adv_train else model.loss
+        loss_syn = loss_fn(_input=img_syn, _label=lab_syn)
+        gw_syn = list(torch.autograd.grad(loss_syn, net_parameters, create_graph=True))
         return gw_syn
 
     def get_real_data(c: int, batch_size: int, **kwargs) -> torch.Tensor:
         try:
             data_real = next(iter_list[c])
-        except StopIteration as e:
+        except StopIteration:
             iter_list[c] = iter(class_loader_list[c])   # don't use itertools.cycle because it's not random
             data_real = next(iter_list[c])
         return model.get_data(data_real)[0][:batch_size]
 
-    # images_all, labels_all = dataset_to_list(train_set)
-    # images_all = torch.stack(images_all).to(env['device'])
-    # labels_all = torch.tensor(labels_all, dtype=torch.long, device=env['device'])
+    # images_all, labels_all = dataset_to_tensor(train_set)
+    # images_all = images_all.to(device=env['device'])
+    # labels_all = labels_all.to(device=env['device'])
     # indices_class = [[] for c in range(dataset.num_classes)]
     # for i, lab in enumerate(labels_all):
     #     indices_class[lab].append(i)
@@ -251,7 +251,10 @@ if __name__ == '__main__':
     # from data import get_images
     # get_real_data = get_images
 
-    if args.init == 'real':
+    if not os.path.exists('./result/dataset_condensation'):
+        os.makedirs('./result/dataset_condensation')
+
+    if kwargs['init'] == 'real':
         bn_size = image_syn.shape[1]
         for c in range(dataset.num_classes):
             image_syn[c].data = get_real_data(c, bn_size)
@@ -259,22 +262,22 @@ if __name__ == '__main__':
     for it in range(Iteration):
         ''' Evaluate synthetic data '''
         # print(f'    {it+1:4d}')
-        if (it + 1) % args.eval_interval == 0:
+        if (it + 1) % kwargs['eval_interval'] == 0:
             if outer_loop > 1:
                 model._validate()
             accs = SmoothedValue(fmt='{global_avg:7.3f} ({min:7.3f}  {max:7.3f})')
             robusts = SmoothedValue(fmt='{global_avg:7.3f} ({min:7.3f}  {max:7.3f})')
             for _ in range(num_eval):
                 # eval_model.load(suffix='')
-                weight_init(eval_model._model)
+                init_weights(eval_model._model)
                 dst_syn_train = TensorDataset(image_syn.detach().clone().flatten(0, 1),
                                               label_syn.detach().clone().flatten(0, 1))
                 loader_train = dataset.get_dataloader(mode='train', pin_memory=False, num_workers=0,
                                                       dataset=dst_syn_train)
                 eval_model._train(loader_train=loader_train, verbose=False, get_data_fn=get_data_fn,
                                   **eval_train_args)
-                result_a, result_b = eval_model._validate(adv_train=model.adv_train, verbose=False)
-                if model.adv_train:
+                result_a, result_b = eval_model._validate(adv_train=bool(model.adv_train), verbose=False)
+                if model.adv_train is not None:
                     acc, robust = result_b - result_a, result_a
                     accs.update(acc)
                     robusts.update(robust)
@@ -291,14 +294,14 @@ if __name__ == '__main__':
                   f'median: {float(image_syn.median()):.2f} '
                   f'({float(image_syn.min()):.2f}, {float(image_syn.max()):.2f})')
             cur_result = robusts.global_avg + accs.global_avg if eval_model.adv_train else accs.global_avg
-            if cur_result > best_result:
+            if cur_result > best_result + 1e-3:
                 best_result = cur_result
                 print(' ' * 12, '{purple}best result update!{reset}'.format(**ansi))
-                if args.file_path:
+                if kwargs['file_path']:
                     data = {'image_syn': image_syn, 'label_syn': label_syn}
-                    torch.save(data, args.file_path)
-                    print(' ' * 12, 'file save at: ', args.file_path)
-                if args.save_img:
+                    torch.save(data, kwargs['file_path'])
+                    print(' ' * 12, 'file save at: ', kwargs['file_path'])
+                if kwargs['save_img']:
                     for c in range(len(image_syn)):
                         for i in range(len(image_syn[c])):
                             filename = f'./result/dataset_condensation/{dataset.name}_{c}_{i}.png'
@@ -306,12 +309,12 @@ if __name__ == '__main__':
                             if dataset.normalize:
                                 image = image.add(mean).mul(std)
                             image = image.clamp(0, 1)
-                            save_tensor_as_img(filename, image)
+                            F.to_pil_image(image).save(filename)
         # model.load(suffix='')
-        weight_init(model._model)
+        init_weights(model._model)
 
         # warmup_args = dict(**train_args)
-        # warmup_args['epoch'] = 3
+        # warmup_args['epochs'] = 3
         # warmup_args['lr_scheduler'] = None
         # warmup_args['adv_train'] = False
         # model._train(verbose=False, change_train_eval=False, **warmup_args)
@@ -353,15 +356,11 @@ if __name__ == '__main__':
                 img_syn = image_syn[c]
                 lab_syn = label_syn[c]
 
-                if args.first_term is not None:
-                    perturbed, _ = model.pgd.optimize(_input=img_syn.detach(), target=lab_syn,
-                                                      **fgsm_args[dataset.name])
-                    img_syn = img_syn + (perturbed - img_syn).detach()
-                gw_syn = get_syn_grad(img_syn, lab_syn)
-                if model.adv_train_free:
+                gw_syn = get_syn_grad(img_syn, lab_syn, adv_train=bool(first_term))
+                if model.adv_train == 'free':
                     gw_real = get_real_grad(img_real, lab_real, adv_train=False)
                     loss += match_loss(gw_syn, gw_real, dis_metric, fim_inv_list=fim_inv_list, kfac=kfac)
-                gw_real = get_real_grad(img_real, lab_real, adv_train=model.adv_train)
+                gw_real = get_real_grad(img_real, lab_real, adv_train=bool(model.adv_train))
                 loss += match_loss(gw_syn, gw_real, dis_metric, fim_inv_list=fim_inv_list, kfac=kfac)
             optimizer_img.zero_grad()
             loss.backward()
@@ -381,8 +380,6 @@ if __name__ == '__main__':
             if ol == outer_loop - 1:
                 break
 
-            # sub_loader = dataset.get_dataloader(mode='train', full=False, seed=random.randint(0, 1000))
-            # model._train(verbose=False, loader_train=sub_loader, **train_args)
             ''' update network '''
             dst_syn_train = TensorDataset(image_syn.detach().clone().flatten(0, 1),
                                           label_syn.detach().clone().flatten(0, 1))
