@@ -1,11 +1,11 @@
 #!/usr/bin/env python3
 
 from .badnet import BadNet
-from trojanvision.utils.model import weight_init
+from trojanzoo.utils.model import init_weights
 from trojanvision.attacks.adv import PGD    # TODO: Need to check whether this will cause ImportError
 from trojanvision.optim import PGDoptimizer
 from trojanvision.environ import env
-from trojanzoo.utils.data import TensorListDataset, dataset_to_list
+from trojanzoo.utils.data import TensorListDataset, dataset_to_tensor, sample_batch
 
 import torch
 import torch.nn as nn
@@ -34,7 +34,7 @@ class CleanLabel(BadNet):
         tau (float): the interpolation constant used to balance source imgs and target imgs. Default: 0.4.
         epsilon (float): the perturbation bound in input space. Default: 0.1.
         noise_dim (int): the dimension of the input in the generator. Default: 100.
-        generator_iters (int): the epoch for training the generator. Default: 1000.
+        generator_iters (int): the epochs for training the generator. Default: 1000.
         critic_iter (int): the critic iterations per generator training iteration. Default: 5.
 
 
@@ -53,18 +53,21 @@ class CleanLabel(BadNet):
     def add_argument(cls, group: argparse._ArgumentGroup):
         super().add_argument(group)
         group.add_argument('--poison_generation_method', choices=['pgd', 'gan'],
-                           help='the chosen method to generate poisoned sample, defaults to config[clean_label][poison_generation_method]="pgd"')
+                           help='the chosen method to generate poisoned sample, '
+                           'defaults to config[clean_label][poison_generation_method]="pgd"')
         group.add_argument('--pgd_alpha', type=float)
         group.add_argument('--pgd_eps', type=float)
         group.add_argument('--pgd_iter', type=int)
         group.add_argument('--tau', type=float,
-                           help='the interpolation constant used to balance source imgs and target imgs, defaults to config[clean_label][tau]=0.2')
+                           help='the interpolation constant used to balance source imgs and target imgs, '
+                           'defaults to config[clean_label][tau]=0.2')
         group.add_argument('--noise_dim', type=int,
-                           help='the dimension of the input in the generator, defaults to config[clean_label][noise_dim]=100')
+                           help='the dimension of the input in the generator, '
+                           'defaults to config[clean_label][noise_dim]=100')
         group.add_argument('--train_gan', action='store_true',
                            help='whether train the GAN if it already exists, defaults to False')
         group.add_argument('--generator_iters', type=int,
-                           help=' the epoch for training the generator, defaults to config[clean_label][generator_iters]=1000')
+                           help=' the epochs for training the generator, defaults to config[clean_label][generator_iters]=1000')
         group.add_argument('--critic_iter', type=int,
                            help=' the critic iterations per generator training iteration, defaults to config[clean_label][critic_iter]=5')
         return group
@@ -104,17 +107,20 @@ class CleanLabel(BadNet):
     def attack(self, optimizer: torch.optim.Optimizer, lr_scheduler: torch.optim.lr_scheduler._LRScheduler, **kwargs):
 
         target_class_set = self.dataset.get_dataset('train', class_list=[self.target_class])
-        target_imgs_list, _ = dataset_to_list(target_class_set)
-        target_imgs = torch.stack(target_imgs_list[:self.poison_num]).to(env['device'])
+        target_imgs, _ = sample_batch(target_class_set, batch_size=self.poison_num)
+        target_imgs = target_imgs.to(env['device'])
 
         full_set = self.dataset.get_dataset('train')
         poison_set: TensorListDataset = None    # TODO
         if self.poison_generation_method == 'pgd':
             poison_label = self.target_class * torch.ones(len(target_imgs), dtype=torch.long, device=target_imgs.device)
-
-            poison_imgs, _ = self.model.remove_misclassify(data=(target_imgs, poison_label))
-            poison_imgs, _ = self.pgd.optimize(_input=poison_imgs)
-            poison_imgs = self.add_mark(poison_imgs).cpu()
+            result = []
+            for data in zip(target_imgs.chunk(self.dataset.batch_size), poison_label.chunk(self.dataset.batch_size)):
+                poison_img, _ = self.model.remove_misclassify(data)
+                poison_img, _ = self.pgd.optimize(poison_img)
+                poison_img = self.add_mark(poison_img).cpu()
+                result.append(poison_img)
+            poison_imgs = torch.cat(result)
 
             poison_set = TensorListDataset(poison_imgs, [self.target_class] * len(poison_imgs))
             # poison_set = torch.utils.data.ConcatDataset([poison_set, target_original_dataset])
@@ -126,12 +132,10 @@ class CleanLabel(BadNet):
             y_list = []
             for source_class in other_classes:
                 print('Process data of Source Class: ', source_class)
-                source_class_dataset = self.dataset.get_dataset(mode='train', full=True, class_list=[source_class])
+                source_class_dataset = self.dataset.get_dataset(mode='train', class_list=[source_class])
                 sample_source_class_dataset, _ = self.dataset.split_dataset(
                     source_class_dataset, self.poison_num)
-                sample_source_class_dataloader = self.dataset.get_dataloader(mode='train', dataset=sample_source_class_dataset,
-                                                                             batch_size=self.poison_num, num_workers=0)
-                source_imgs, _ = self.model.get_data(next(iter(sample_source_class_dataloader)))
+                source_imgs = dataset_to_tensor(sample_source_class_dataset)[0].to(device=env['device'])
 
                 g_path = f'{self.folder_path}gan_dim{self.noise_dim}_class{source_class}_g.pth'
                 d_path = f'{self.folder_path}gan_dim{self.noise_dim}_class{source_class}_d.pth'
@@ -150,25 +154,28 @@ class CleanLabel(BadNet):
                     torch.save(self.wgan.D.state_dict(), d_path)
                     print(f'GAN Model Saved at : \n{g_path}\n{d_path}')
                     continue
-                source_encode = self.wgan.get_encode_value(source_imgs).detach()
-                target_encode = self.wgan.get_encode_value(target_imgs).detach()
-                # noise = torch.randn_like(source_encode)
-                # from trojanzoo.utils.tensor import save_tensor_as_img
-                # source_img = self.wgan.G(source_encode)
-                # target_img = self.wgan.G(target_encode)
-                # for i in range(len(source_img)):
-                #     save_tensor_as_img(f'./imgs/source_{i}.png', source_img[i])
-                # for i in range(len(target_img)):
-                #     save_tensor_as_img(f'./imgs/target_{i}.png', target_img[i])
-                # exit()
-                interpolation_encode = source_encode * self.tau + target_encode * (1 - self.tau)
-                poison_imgs = self.wgan.G(interpolation_encode).detach()
-                poison_imgs = self.add_mark(poison_imgs)
 
-                poison_label = [self.target_class] * len(poison_imgs)
-                poison_imgs = poison_imgs.cpu()
-                x_list.append(poison_imgs)
-                y_list.extend(poison_label)
+                for source_chunk, target_chunk in zip(source_imgs.chunk(self.dataset.batch_size),
+                                                      target_imgs.chunk(self.dataset.batch_size)):
+                    source_encode = self.wgan.get_encode_value(source_chunk).detach()
+                    target_encode = self.wgan.get_encode_value(target_chunk).detach()
+                    # noise = torch.randn_like(source_encode)
+                    # source_img = self.wgan.G(source_encode)
+                    # target_img = self.wgan.G(target_encode)
+                    # if not os.path.exists('./imgs'):
+                    #     os.makedirs('./imgs')
+                    # for i in range(len(source_img)):
+                    #     F.to_pil_image(source_img[i]).save(f'./imgs/source_{i}.png')
+                    # for i in range(len(target_img)):
+                    #     F.to_pil_image(target_img[i]).save(f'./imgs/target_{i}.png')
+                    # exit()
+                    interpolation_encode = source_encode * self.tau + target_encode * (1 - self.tau)
+                    poison_imgs = self.wgan.G(interpolation_encode).detach()
+                    poison_imgs = self.add_mark(poison_imgs)
+
+                    poison_imgs = poison_imgs.cpu()
+                    x_list.append(poison_imgs)
+                y_list.extend([self.target_class] * len(source_imgs))
             assert not self.train_gan
             x_list = torch.cat(x_list)
             poison_set = TensorListDataset(x_list, y_list)
@@ -259,8 +266,8 @@ class WGAN(object):
         self.gan_pgd = PGDoptimizer(pgd_eps=1.0, iteration=500, output=0)
 
     def reset_parameters(self):
-        self.G.apply(weight_init)
-        self.D.apply(weight_init)
+        init_weights(self.G)
+        init_weights(self.D)
 
     def train(self, train_dataloader):
         self.g_optimizer.zero_grad()
@@ -310,11 +317,11 @@ class WGAN(object):
         """According to the image and Generator, utilize pgd optimization to get the d dimension encoding value.
 
         Args:
-            imgs (torch.FloatTensor): the chosen image to get its encoding value, also considered as the output of Generator.
+            imgs (torch.Tensor): the chosen image to get its encoding value, also considered as the output of Generator.
             noise_dim (int): the dimension of the input in the generator.
 
         Returns:
-            torch.FloatTensor: the synthesized poisoned image.
+            torch.Tensor: the synthesized poisoned image.
         """
 
         def loss_func(X: torch.Tensor):
