@@ -1,4 +1,4 @@
 #!/usr/bin/env python3
 
-from .term_study import TermStudy
+# from .term_study import TermStudy
 from .unlearn import Unlearn
