@@ -2,10 +2,8 @@
 
 from .poison_basic import PoisonBasic
 from trojanvision.attacks import PGD
-from trojanvision.defenses.adv.curvature import Curvature
 from trojanvision.optim import PGDoptimizer
 from trojanvision.models import MagNet
-from trojanzoo.utils import to_list
 
 import torch
 import numpy as np
@@ -44,12 +42,13 @@ class IMC_Poison(PoisonBasic):
                                 iteration=self.pgd_iter)
         self.stop_conf: float = stop_conf
         if magnet:
-            self.magnet: MagNet = MagNet(dataset=self.dataset, pretrain=True)
+            self.magnet: MagNet = MagNet(dataset=self.dataset, pretrained=True)
         self.randomized_smooth: bool = randomized_smooth
         if curvature:
-            self.curvature: Curvature = Curvature(model=self.model)
+            from trojanvision.defenses.adv.curvature import Curvature
+            self.curvature = Curvature(model=self.model)
 
-    def attack(self, epoch: int, **kwargs):
+    def attack(self, epochs: int, **kwargs):
         # model._validate()
         total = 0
         target_conf_list = []
@@ -69,7 +68,7 @@ class IMC_Poison(PoisonBasic):
         easy = 0
         difficult = 0
         normal = 0
-        loader = self.dataset.get_dataloader(mode='valid', batch_size=self.dataset.test_batch_size)
+        loader = self.dataset.get_dataloader(mode='valid', batch_size=1)
         if 'curvature' in self.__dict__.keys():
             benign_curvature = self.curvature.benign_measure()
             tgt_curvature_list = []
@@ -101,7 +100,7 @@ class IMC_Poison(PoisonBasic):
             normal += 1
             target_conf, target_acc, clean_acc = self.validate_fn()
             noise = torch.zeros_like(_input)
-            poison_input = self.optimize(_input=_input, _label=target_label, epoch=epoch, noise=noise, **kwargs)
+            poison_input = self.optimize(_input=_input, _label=target_label, epochs=epochs, noise=noise, **kwargs)
             pgd_norm = float(noise.norm(p=float('inf')))
             target_conf, target_acc, clean_acc = self.validate_fn()
             target_conf_list.append(target_conf)
@@ -116,8 +115,8 @@ class IMC_Poison(PoisonBasic):
             org_conf = self.model.get_target_prob(_input=poison_input, target=_label)
             tgt_conf = self.model.get_target_prob(_input=poison_input, target=target_label)
             if 'curvature' in self.__dict__.keys():
-                org_curvature_list.extend(to_list(self.curvature.measure(poison_input, _label)))    # type: ignore
-                tgt_curvature_list.extend(to_list(self.curvature.measure(poison_input, target_label)))    # type: ignore
+                org_curvature_list.extend(self.curvature.measure(poison_input, _label).detach().cpu().tolist())
+                tgt_curvature_list.extend(self.curvature.measure(poison_input, target_label).detach().cpu().tolist())
                 print('Curvature:')
                 print(f'    org_curvature: {ks_2samp(org_curvature_list, benign_curvature)}')    # type: ignore
                 print(f'    tgt_curvature: {ks_2samp(tgt_curvature_list, benign_curvature)}')    # type: ignore
@@ -127,11 +126,11 @@ class IMC_Poison(PoisonBasic):
                 tgt_new = self.model.get_target_prob(_input=poison_input, target=target_label, randomized_smooth=True)
                 org_increase = (org_new - org_conf).clamp(min=0.0)
                 tgt_decrease = (tgt_new - tgt_conf).clamp(min=0.0)
-                org_conf_list.extend(to_list(org_increase))    # type: ignore
-                tgt_conf_list.extend(to_list(tgt_decrease))    # type: ignore
+                org_conf_list.extend(org_increase.detach().cpu().tolist())
+                tgt_conf_list.extend(tgt_decrease.detach().cpu().tolist())
                 print('Randomized Smooth:')
-                print(f'    org_confidence: {np.mean(org_conf_list)}')    # type: ignore
-                print(f'    tgt_confidence: {np.mean(tgt_conf_list)}')    # type: ignore
+                print(f'    org_confidence: {np.mean(org_conf_list)}')
+                print(f'    tgt_confidence: {np.mean(tgt_conf_list)}')
                 print()
             if 'magnet' in self.__dict__.keys():
                 poison_input = self.magnet(poison_input)
@@ -139,11 +138,11 @@ class IMC_Poison(PoisonBasic):
                 tgt_new = self.model.get_target_prob(_input=poison_input, target=target_label)
                 org_increase = (org_new - org_conf).clamp(min=0.0)
                 tgt_decrease = (tgt_conf - tgt_new).clamp(min=0.0)
-                org_magnet_list.extend(to_list(org_increase))    # type: ignore
-                tgt_magnet_list.extend(to_list(tgt_decrease))    # type: ignore
+                org_magnet_list.extend(org_increase.detach().cpu().tolist())
+                tgt_magnet_list.extend(tgt_decrease.detach().cpu().tolist())
                 print('MagNet:')
-                print(f'    org_confidence: {np.mean(org_magnet_list)}')    # type: ignore
-                print(f'    tgt_confidence: {np.mean(tgt_magnet_list)}')    # type: ignore
+                print(f'    org_confidence: {np.mean(org_magnet_list)}')
+                print(f'    tgt_confidence: {np.mean(tgt_magnet_list)}')
                 print()
             total += 1
 
